import { createContext, useContext, useMemo } from 'react';
import { GraphqlQueryStore, GraphqlWatchQueryStore } from '@proscom/prostore-apollo';
import { useRequestStore } from '@proscom/prostore-react';

var ApolloClientsContext = createContext(null);
var ApolloClientsManager = /** @class */ (function () {
    function ApolloClientsManager(clients) {
        this.clients = clients;
    }
    ApolloClientsManager.prototype.getClient = function (key) {
        if (key === void 0) { key = 'default'; }
        return this.clients[key];
    };
    ApolloClientsManager.prototype.extract = function () {
        var result = {};
        for (var _i = 0, _a = Object.entries(this.clients); _i < _a.length; _i++) {
            var _b = _a[_i], key = _b[0], value = _b[1];
            result[key] = value.extract();
        }
        return result;
    };
    return ApolloClientsManager;
}());
function useContextApolloClient(clientOrName) {
    var apolloClientsContext = useContext(ApolloClientsContext);
    if (!clientOrName) {
        return apolloClientsContext.getClient();
    }
    else if (typeof clientOrName === 'string') {
        return apolloClientsContext.getClient(clientOrName);
    }
    else {
        return clientOrName;
    }
}

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function __rest(s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
}

function useGraphqlQuery(_a) {
    var client = _a.client, queryOptions = _a.queryOptions, variables = _a.variables;
    var actualClient = useContextApolloClient(client);
    var store = useMemo(function () {
        return new GraphqlQueryStore(__assign({ client: actualClient }, queryOptions));
    }, [actualClient, queryOptions]);
    return useRequestStore(store, variables);
}
function useGraphqlQueryVars(_a) {
    var variableCreator = _a.variableCreator, queryProps = __rest(_a, ["variableCreator"]);
    var variables = useMemo.apply(void 0, variableCreator);
    return useGraphqlQuery(__assign({}, queryProps, { variables: variables }));
}

/**
 * Создает graphql watchQuery и подписывается на него
 * @param client - Apollo Client instance
 * @param queryOptions - Параметры запроса
 * @param variables - Переменные запроса
 * @returns {{load, check, state}} - Состояние запроса
 */
function useGraphqlWatchQuery(_a) {
    var client = _a.client, queryOptions = _a.queryOptions, variables = _a.variables;
    var actualClient = useContextApolloClient(client);
    var store = useMemo(function () {
        return new GraphqlWatchQueryStore(__assign({ client: actualClient }, queryOptions));
    }, [actualClient, queryOptions]);
    return useRequestStore(store, variables);
}
function useGraphqlWatchQueryVars(_a) {
    var variableCreator = _a.variableCreator, queryProps = __rest(_a, ["variableCreator"]);
    var variables = useMemo.apply(void 0, variableCreator);
    return useGraphqlWatchQuery(__assign({}, queryProps, { variables: variables }));
}

export { ApolloClientsContext, ApolloClientsManager, useContextApolloClient, useGraphqlQuery, useGraphqlQueryVars, useGraphqlWatchQuery, useGraphqlWatchQueryVars };
